diff --git a/dlls/wininet/internet.c b/dlls/wininet/internet.c
index f52103d..28fa5a6 100644
--- a/dlls/wininet/internet.c
+++ b/dlls/wininet/internet.c
@@ -6,6 +6,7 @@
  * Copyright 2002 Jaco Greeff
  * Copyright 2002 TransGaming Technologies Inc.
  * Copyright 2004 Mike McCormack for CodeWeavers
+ * Copyright 2006 Vijay Kiran Kamuju
  *
  * Ulrich Czekalla
  * Aric Stewart
@@ -2239,6 +2240,60 @@ static BOOL INET_QueryOptionHelper(BOOL 
             *lpdwBufferLength = sizeof(ULONG);
             break;
         }
+        case INTERNET_OPTION_USER_AGENT:
+        {
+            LPWININETAPPINFOW lpwai = (LPWININETAPPINFOW)lpwhh;
+            DWORD len=0;
+
+            if (!lpwhh || lpwhh->htype != INTERNET_HANDLE_TYPE_INTERNET)
+            {
+                WARN("Invalid hInternet handle\n");
+                INTERNET_SetLastError(ERROR_INTERNET_INCORRECT_HANDLE_TYPE);
+                *lpdwBufferLength=len;
+                return FALSE;
+            }
+
+            len=WideCharToMultiByte(CP_ACP, 0, lpwai->lpszAgent,-1,NULL,0,NULL,NULL);
+            if(*lpdwBufferLength < len)
+            {
+                INTERNET_SetLastError(ERROR_INSUFFICIENT_BUFFER);
+                if(bIsUnicode)
+                   *lpdwBufferLength=len*2;
+                else
+                   *lpdwBufferLength=len;
+                return FALSE;
+            }
+            *lpdwBufferLength=len;
+
+            if(lpwai->lpszAgent)
+            {
+                if(lpBuffer != NULL)
+                {
+                    --(*lpdwBufferLength);
+                    bSuccess = TRUE;
+                    if(bIsUnicode)
+                    {
+                        memcpy( lpBuffer,lpwai->lpszAgent, (*lpdwBufferLength+1)*sizeof(WCHAR));
+                        //lstrcpynW( lpwai->lpszAgent, lpBuffer, *lpdwBufferLength+1); 
+                        lpBuffer="\0";
+                    }
+                    else
+                    {
+                        WideCharToMultiByte(CP_ACP, 0, lpwai->lpszAgent, -1, (LPSTR)lpBuffer, (*lpdwBufferLength)+1,NULL,NULL);
+                    }
+                }
+                else
+                    INTERNET_SetLastError(ERROR_INSUFFICIENT_BUFFER);
+            }
+            else
+            {
+                *lpdwBufferLength=1;
+                if(lpBuffer != NULL)
+                    --(*lpdwBufferLength);
+                INTERNET_SetLastError(ERROR_INSUFFICIENT_BUFFER);
+            }
+            break;
+        }
         case INTERNET_OPTION_SECURITY_FLAGS:
             FIXME("INTERNET_OPTION_SECURITY_FLAGS: Stub\n");
             break;
@@ -2382,7 +2437,10 @@ BOOL WINAPI InternetSetOptionW(HINTERNET
 
     lpwhh = (LPWININETHANDLEHEADER) WININET_GetObject( hInternet );
     if( !lpwhh )
+    {
+        SetLastError(ERROR_INTERNET_INCORRECT_HANDLE_TYPE);
         return FALSE;
+    }
 
     switch (dwOption)
     {
@@ -2455,6 +2513,13 @@ BOOL WINAPI InternetSetOptionW(HINTERNET
     case INTERNET_OPTION_CONNECT_RETRIES:
         FIXME("Option INTERNET_OPTION_CONNECT_RETRIES: STUB\n");
         break;
+    case INTERNET_OPTION_USER_AGENT:
+    {
+        LPWININETAPPINFOW lpwai = (LPWININETAPPINFOW)lpwhh;
+        //memcpy(lpwai->lpszAgent,lpBuffer,(dwBufferLength+1)*sizeof(WCHAR));
+        lpwai->lpszAgent=lpBuffer;
+        break;
+    }
     default:
         FIXME("Option %ld STUB\n",dwOption);
         INTERNET_SetLastError(ERROR_INVALID_PARAMETER);
@@ -2484,6 +2549,12 @@ BOOL WINAPI InternetSetOptionA(HINTERNET
     DWORD wlen;
     BOOL r;
 
+    if( !hInternet )
+    {
+        SetLastError(ERROR_INTERNET_INCORRECT_HANDLE_TYPE);
+        return FALSE;
+    }
+
     switch( dwOption )
     {
     case INTERNET_OPTION_PROXY:
